@@ -1,9 +1,7 @@
 ### Title: Compute more fit indices
-### Authors: Terrence D. Jorgensen <TJorgensen314@gmail.com>
-###          Sunthud Pornprasertmanit <psunthud@ku.edu>,
-###          Aaron Boulton <aboulton@ku.edu>,
-###          Ruben Arslan <rubenarslan@gmail.com>
-### Last updated: 10 January 2021
+### Authors: Terrence D. Jorgensen, Sunthud Pornprasertmanit,
+###          Aaron Boulton, Ruben Arslan, Mauricio Garnier-Villarreal
+### Last updated: 9 February 2026
 ### Description: Calculations for promising alternative fit indices
 
 
@@ -12,119 +10,192 @@
 ##'
 ##' Calculate more fit indices that are not already provided in lavaan.
 ##'
-##' Gamma Hat (gammaHat; West, Taylor, & Wu, 2012) is a global fit index which
-##' can be computed (assuming equal number of indicators across groups) by
+##' See [nullRMSEA()] for the further details of the computation of
+##' RMSEA of the null model.
 ##'
-##' \deqn{ gammaHat =\frac{p}{p + 2 \times \frac{\chi^{2}_{k} - df_{k}}{N}} ,}
+##' Gamma-Hat (`gammaHat`; West, Taylor, & Wu, 2012) is a global
+##' goodness-of-fit index which can be computed (assuming equal number of
+##' indicators across groups) by
+##'
+##'   \deqn{ \hat{\Gamma} =\frac{p}{p + 2 \times \frac{\chi^{2}_{k} - df_{k}}{N}},}
 ##'
 ##' where \eqn{p} is the number of variables in the model, \eqn{\chi^{2}_{k}} is
 ##' the \eqn{\chi^2} test statistic value of the target model, \eqn{df_{k}} is
 ##' the degree of freedom when fitting the target model, and \eqn{N} is the
-##' sample size (or sample size minus the number of groups if \code{mimic} is
-##' set to \code{"EQS"}).
+##' sample size (or sample size minus the number of groups if `mimic` is
+##' set to `"EQS"`).
 ##'
-##' Adjusted Gamma Hat (adjGammaHat; West, Taylor, & Wu, 2012) is a global fit
-##' index which can be computed by
+##' Adjusted Gamma-Hat (`adjGammaHat`; West, Taylor, & Wu, 2012) is a
+##' global fit index which can be computed by
 ##'
-##' \deqn{ adjGammaHat = \left(1 - \frac{K \times p \times (p + 1)}{2 \times
-##' df_{k}} \right) \times \left( 1 - gammaHat \right) ,}
+##'   \deqn{ \hat{\Gamma}_\textrm{adj} = \left(1 - \frac{K \times p \times
+##'   (p + 1)}{2 \times df_{k}} \right) \times \left( 1 - \hat{\Gamma} \right),}
 ##'
-##' where \eqn{K} is the number of groups (please refer to Dudgeon, 2004 for the
-##' multiple-group adjustment for agfi*).
+##' where \eqn{K} is the number of groups (please refer to Dudgeon, 2004, for
+##' the multiple-group adjustment for `adjGammaHat`).
 ##'
-##' Corrected Akaike Information Criterion (aic.smallN; Burnham & Anderson,
-##' 2003) is a corrected version of AIC for small sample size, often abbreviated
-##' AICc:
-##'
-##' \deqn{ aic.smallN = AIC + \frac{2k(k + 1)}{N - k - 1},}
-##'
-##' where \eqn{AIC} is the original AIC: \eqn{-2 \times LL + 2k} (where \eqn{k}
-##' = the number of estimated parameters in the target model). Note that AICc is
-##' a small-sample correction derived for univariate regression models, so it is
-##' probably \emph{not} appropriate for comparing SEMs.
-##'
-##' Corrected Bayesian Information Criterion (bic.priorN; Kuha, 2004) is similar
-##' to BIC but explicitly specifying the sample size on which the prior is based
-##' (\eqn{N_{prior}}).
-##'
-##' \deqn{ bic.priorN = f + k\log{(1 + N/N_{prior})},}
-##'
-##' Stochastic information criterion (SIC; Preacher, 2006) is similar to AIC or
-##' BIC. This index will account for model complexity in the model's function
-##' form, in addition to the number of free parameters. This index will be
-##' provided only when the \eqn{\chi^2} value is not scaled. The SIC can be
-##' computed by
-##'
-##' \deqn{ sic = \frac{1}{2}\left(f - \log{\det{I(\hat{\theta})}}\right),}
-##'
-##' where \eqn{I(\hat{\theta})} is the information matrix of the parameters.
-##'
-##' Hannan-Quinn Information Criterion (hqc; Hannan & Quinn, 1979) is used for
-##' model selection similar to AIC or BIC.
-##'
-##' \deqn{ hqc = f + 2k\log{(\log{N})},}
-##'
-##' Note that if Satorra--Bentler or Yuan--Bentler's method is used, the fit
+##' Note that if Satorra--Bentler's or Yuan--Bentler's method is used, the fit
 ##' indices using the scaled \eqn{\chi^2} values are also provided.
 ##'
-##' See \code{\link{nullRMSEA}} for the further details of the computation of
-##' RMSEA of the null model.
+##' The remaining indices are information criteria calculated using the
+##' `object`'s \eqn{-2 \times} log-likelihood, abbreviated \eqn{-2LL}.
+##'
+##' Corrected Akaike Information Criterion (`aic.smallN`; Burnham &
+##' Anderson, 2003) is a corrected version of AIC for small sample size, often
+##' abbreviated AICc:
+##'
+##'   \deqn{ \textrm{AIC}_{\textrm{small}-N} = AIC + \frac{2q(q + 1)}{N - q - 1},}
+##'
+##' where \eqn{AIC} is the original AIC: \eqn{-2LL + 2q} (where \eqn{q}
+##' = the number of estimated parameters in the target model). Note that AICc is
+##' a small-sample correction derived for univariate regression models, so it is
+##' probably *not* appropriate for comparing SEMs.
+##'
+##' Corrected Bayesian Information Criterion (`bic.priorN`; Kuha, 2004) is
+##' similar to BIC but explicitly specifying the sample size on which the prior
+##' is based (\eqn{N_{prior}}) using the `nPrior` argument.
+##'
+##'   \deqn{ \textrm{BIC}_{\textrm{prior}-N} = -2LL + q\log{( 1 + \frac{N}{N_{prior}} )}.}
+##'
+##' Bollen et al. (2012, 2014) discussed additional BICs that incorporate more
+##' terms from a Taylor series expansion, which the standard BIC drops.  The
+##' "Scaled Unit-Information Prior" BIC is calculated depending on whether the
+##' product of the vector of estimated model parameters (\eqn{\hat{\theta}}) and
+##' the observed information matrix (FIM) exceeds the number of estimated model
+##' parameters (Case 1) or not (Case 2), which is checked internally:
+##'
+##'   \deqn{ \textrm{SPBIC}_{\textrm{Case 1}} = -2LL + q(1 - \frac{q}{\hat{\theta}^{'} \textrm{FIM} \hat{\theta}}), \textrm{ or}}
+##'   \deqn{ \textrm{SPBIC}_{\textrm{Case 2}} = -2LL + \hat{\theta}^{'} \textrm{FIM} \hat{\theta},}
+##'
+##' Note that this implementation of SPBIC is calculated on the assumption that
+##' priors for all estimated parameters are centered at zero, which is
+##' inappropriate for most SEMs (e.g., variances should not have priors centered
+##' at the lowest possible value; Bollen, 2014, p. 6).
+##'
+##' Bollen et al. (2014, eq. 14) credit the HBIC to Haughton (1988):
+##'
+##'   \deqn{ \textrm{HBIC} = -2LL + q\log{\frac{N}{2 \pi}}.}
+##'
+##' Bollen et al. (2012, p. 305) proposed the information matrix (\eqn{I})-based BIC by
+##' adding another term:
+##'
+##'   \deqn{ \textrm{IBIC} = -2LL + q\log{\frac{N}{2 \pi}} + \log{\det{\textrm{FIM}}},}
+##'
+##' or equivalently, using the inverse information (the asymptotic sampling
+##' covariance matrix of estimated parameters: ACOV):
+##'
+##'   \deqn{ \textrm{IBIC} = -2LL - q\log{2 \pi} - \log{\det{\textrm{ACOV}}}.}
+##'
+##' Stochastic information criterion (SIC; see Preacher, 2006, for details) is
+##' similar to IBIC but does not include the \eqn{q\log{2 \pi}} term that is
+##' also in HBIC.  SIC and IBIC both account for model complexity in a model's
+##' functional form, not merely the number of free parameters.  The SIC can be
+##' computed as:
+##'
+##'   \deqn{ \textrm{SIC} = -2LL + q\log{N} + \log{\det{\textrm{FIM}}} = -2LL - \log{\det{\textrm{ACOV}}}.}
+##'
+##' Hannan--Quinn Information Criterion (HQC; Hannan & Quinn, 1979) is used for
+##' model selection, similar to AIC or BIC.
+##'
+##' \deqn{ \textrm{HQC} = -2LL + 2q\log{(\log{N})},}
+##'
+##' Bozdogan Information Complexity (ICOMP) Criteria (Howe et al., 2011),
+##' instead of penalizing the number of free parameters directly,
+##' ICOMP penalizes the covariance complexity of the model.
+##'
+##' \deqn{ \textrm{ICOMP} = -2LL + s \times log(\frac{\bar{\lambda_a}}{\bar{\lambda_g}}) }
 ##'
 ##'
 ##' @importFrom lavaan lavInspect
 ##'
-##' @param object The lavaan model object provided after running the \code{cfa},
-##' \code{sem}, \code{growth}, or \code{lavaan} functions.
+##' @param object The lavaan model object provided after running the `cfa`,
+##'   `sem`, `growth`, or `lavaan` functions.
 ##' @param fit.measures Additional fit measures to be calculated. All additional
-##' fit measures are calculated by default
+##'   fit measures are calculated by default
 ##' @param nPrior The sample size on which prior is based. This argument is used
-##' to compute BIC*.
-##' @return \enumerate{
-##'  \item \code{gammaHat}: Gamma Hat
-##'  \item \code{adjGammaHat}: Adjusted Gamma Hat
-##'  \item \code{baseline.rmsea}: RMSEA of the Baseline (Null) Model
-##'  \item \code{aic.smallN}: Corrected (for small sample size) Akaike Information Criterion
-##'  \item \code{bic.priorN}: Bayesian Information Criterion with specified prior sample size
-##'  \item \code{sic}: Stochastic Information Criterion
-##'  \item \code{hqc}: Hannan-Quinn Information Criterion
-##'  \item \code{gammaHat.scaled}: Gamma Hat using scaled \eqn{\chi^2}
-##'  \item \code{adjGammaHat.scaled}: Adjusted Gamma Hat using scaled \eqn{\chi^2}
-##'  \item \code{baseline.rmsea.scaled}: RMSEA of the Baseline (Null) Model using scaled \eqn{\chi^2}
+##'   to compute `bic.priorN`.
+##'
+##' @return A `numeric` `lavaan.vector` including any of the
+##'   following requested via `fit.measures=`
+##' \enumerate{
+##'  \item `gammaHat`: Gamma-Hat
+##'  \item `adjGammaHat`: Adjusted Gamma-Hat
+##'  \item `baseline.rmsea`: RMSEA of the default baseline (i.e., independence) model
+##'  \item `gammaHat.scaled`: Gamma-Hat using scaled \eqn{\chi^2}
+##'  \item `adjGammaHat.scaled`: Adjusted Gamma-Hat using scaled \eqn{\chi^2}
+##'  \item `baseline.rmsea.scaled`: RMSEA of the default baseline (i.e.,
+##'        independence) model using scaled \eqn{\chi^2}
+##'  \item `aic.smallN`: Corrected (for small sample size) AIC
+##'  \item `bic.priorN`: BIC with specified prior sample size
+##'  \item `spbic`: Scaled Unit-Information Prior BIC (SPBIC)
+##'  \item `hbic`: Haughton's BIC (HBIC)
+##'  \item `ibic`: Information-matrix-based BIC (IBIC)
+##'  \item `sic`: Stochastic Information Criterion (SIC)
+##'  \item `hqc`: Hannan-Quinn Information Criterion (HQC)
+##'  \item `icomp`: Bozdogan Information Complexity (ICOMP) Criteria
 ##' }
+##'
 ##' @author Sunthud Pornprasertmanit (\email{psunthud@@gmail.com})
 ##'
 ##' Terrence D. Jorgensen (University of Amsterdam; \email{TJorgensen314@@gmail.com})
 ##'
-##' Aaron Boulton (University of North Carolina, Chapel Hill; \email{aboulton@@email.unc.edu})
+##' Aaron Boulton (University of Delaware)
 ##'
 ##' Ruben Arslan (Humboldt-University of Berlin, \email{rubenarslan@@gmail.com})
 ##'
 ##' Yves Rosseel (Ghent University; \email{Yves.Rosseel@@UGent.be})
 ##'
-##' @seealso \itemize{ \item \code{\link{miPowerFit}} For the modification
-##' indices and their power approach for model fit evaluation \item
-##' \code{\link{nullRMSEA}} For RMSEA of the null model }
+##' Mauricio Garnier-Villarreal (Vrije Universiteit Amsterdam; \email{mgv@pm.me})
 ##'
-##' @references Burnham, K., & Anderson, D. (2003). \emph{Model selection and
-##' multimodel inference: A practical--theoretic approach}. New York, NY:
+##' A great deal of feedback was provided by Kris Preacher regarding Bollen et
+##' al.'s (2012, 2014) extensions of BIC.
+##'
+##' @seealso
+##' \itemize{
+##' \item [epcEquivFit()] For the equivalence testing based on expected
+##'   parameter changes for model fit evaluation
+##' \item [nullRMSEA()] For RMSEA of the default independence model
+##' }
+##'
+##' @references
+##'
+##' Bollen, K. A., Ray, S., Zavisca, J., & Harden, J. J. (2012). A comparison of
+##' Bayes factor approximation methods including two new methods.
+##' *Sociological Methods & Research, 41*(2), 294--324.
+##' \doi{10.1177/0049124112452393}
+##'
+##' Bollen, K. A., Harden, J. J., Ray, S., & Zavisca, J. (2014). BIC and
+##' alternative Bayesian information criteria in the selection of structural
+##' equation models. *Structural Equation Modeling, 21*(1), 1--19.
+##' \doi{10.1080/10705511.2014.856691}
+##'
+##' Burnham, K., & Anderson, D. (2003). *Model selection and
+##' multimodel inference: A practical--theoretic approach*. New York, NY:
 ##' Springer--Verlag.
 ##'
 ##' Dudgeon, P. (2004). A note on extending Steiger's (1998) multiple sample
 ##' RMSEA adjustment to other noncentrality parameter-based statistic.
-##' \emph{Structural Equation Modeling, 11}(3), 305--319.
+##' *Structural Equation Modeling, 11*(3), 305--319.
 ##' \doi{10.1207/s15328007sem1103_1}
 ##'
+##' Howe, E. D., Bozdogan, H., & Katragadda, S. (2011). Structural equation
+##' modeling (SEM) of categorical and mixed-data using the novel Gifi
+##' transformations and information complexity (ICOMP) criterion.
+##' *Istanbul University Journal of the School of Business Administration, 40*(1), 86--123.
+##'
 ##' Kuha, J. (2004). AIC and BIC: Comparisons of assumptions and performance.
-##' \emph{Sociological Methods Research, 33}(2), 188--229.
+##' *Sociological Methods Research, 33*(2), 188--229.
 ##' \doi{10.1177/0049124103262065}
 ##'
 ##' Preacher, K. J. (2006). Quantifying parsimony in structural equation
-##' modeling. \emph{Multivariate Behavioral Research, 43}(3), 227-259.
+##' modeling. *Multivariate Behavioral Research, 43*(3), 227--259.
 ##' \doi{10.1207/s15327906mbr4103_1}
 ##'
 ##' West, S. G., Taylor, A. B., & Wu, W. (2012). Model fit and model selection
-##' in structural equation modeling. In R. H. Hoyle (Ed.), \emph{Handbook of
-##' Structural Equation Modeling} (pp. 209--231). New York, NY: Guilford.
+##' in structural equation modeling. In R. H. Hoyle (Ed.), *Handbook of
+##' structural equation modeling* (pp. 209--231). New York, NY: Guilford.
+##'
+##'
 ##' @examples
 ##'
 ##' HS.model <- ' visual  =~ x1 + x2 + x3
@@ -142,7 +213,7 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
   ## check for validity of user-specified "fit.measures" argument
   fit.choices <- c("gammaHat","adjGammaHat","baseline.rmsea",
                    "gammaHat.scaled","adjGammaHat.scaled","baseline.rmsea.scaled",
-                   "aic.smallN","bic.priorN","hqc","sic")
+                   "aic.smallN","bic.priorN","spbic","hbic","ibic","sic","hqc","icomp")
   flags <- setdiff(fit.measures, c("all", fit.choices))
   if (length(flags)) stop(paste("Argument 'fit.measures' includes invalid options:",
                                 paste(flags, collapse = ", "),
@@ -163,10 +234,10 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
   N <- n <- lavInspect(object, "ntotal")
   if (lavInspect(object, "options")$mimic == "EQS") n <- n - ngroup
 
-  # Calculate -2*log(likelihood)
+  ## Calculate -2*log(likelihood)
   f <- -2 * fit["logl"]
 
-  # Compute fit indices
+  ## Compute fit indices
   result <- list()
   if (length(grep("gamma", fit.measures, ignore.case = TRUE))) {
     gammaHat <- p / (p + 2 * ((fit["chisq"] - fit["df"]) / n))
@@ -186,6 +257,8 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
       result["baseline.rmsea.scaled"] <- nullRMSEA(object, scaled = TRUE, silent = TRUE)
     }
   }
+
+  ## Compute information criteria
   if (!is.na(f)) {
     if ("aic.smallN" %in% fit.measures) {
       warning('AICc (aic.smallN) was developed for univariate linear models.',
@@ -195,11 +268,60 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
     if ("bic.priorN" %in% fit.measures) {
       result["bic.priorN"] <- f + log(1 + N/nPrior) * nParam
     }
-    if ("hqc" %in% fit.measures) result["hqc"] <- f + 2 * log(log(N)) * nParam
-    if ("sic" %in% fit.measures) result["sic"] <- sic(f, object)
+    if ("spbic" %in% fit.measures) {
+      theta <- lavaan::coef(object)
+      FIM <- lavInspect(object, "information.observed")
+      junk <- t(theta) %*% FIM %*% theta
+      if (nParam < junk) {
+        result["spbic"] <- f + nParam*(1 - log(nParam / junk)) # Case 1
+      } else result["spbic"] <- f + junk # Case 2
+    }
+    if ("hbic" %in% fit.measures) result["hbic"] <- f + nParam*log(N/(2*pi))
+    if ("icomp" %in% fit.measures) {
+      Fhatinv <- lavInspect(object, "inverted.information.expected")
+      s <- qr(Fhatinv)$rank
+      C1 <- (s/2)*log((sum(diag(Fhatinv)))/s) - .5*log(det(Fhatinv))
+      result["icomp"] <- f + 2*C1
+    }
+
+    ## check determinant of ACOV for IBIC and SIC
+    if (any(c("ibic","sic") %in% fit.measures)) {
+      ACOV <- lavInspect(object, "vcov")
+      detACOV <- det(ACOV)
+      if (detACOV <= 0) {
+        ## look for duplicate names (simple equality constraints) to
+        ## remove any obviously redundant parameters
+        #TODO: any way to check for redundancies implied by "==" constraints?
+        RN <- unique(rownames(ACOV))
+        if (length(RN) < nrow(ACOV)) detACOV <- det(ACOV[RN, RN])
+      }
+    }
+    if ("ibic" %in% fit.measures) {
+      if (detACOV <= 0) {
+        result["ibic"] <- NA
+        message('Determinant of vcov(object) <= 0, so IBIC cannot be calculated')
+      } else result["ibic"] <- f - nParam*log(2*pi) - log(detACOV)
+    }
+    if ("sic" %in% fit.measures) {
+      if (detACOV <= 0) {
+        result["sic"] <- NA
+        message('Determinant of vcov(object) <= 0, so SIC cannot be calculated')
+      } else {
+        result["sic"] <- f - log(detACOV)
+        ## doi:10.1007/s10519-004-5587-0 (p. 596) says to use observed Fisher information,
+        ## but ACOV will be lavInspect(object, "options")$information
+        ## legacy code:
+        # E.inv <- lavaan::lavTech(object, "inverted.information.observed")
+        # E <- MASS::ginv(E.inv) * lavaan::nobs(object) # multiply unit information by N
+      }
+    }
+
+    if ("hqc" %in% fit.measures) result["hqc"] <- f + 2*nParam*log(log(N))
   }
+
+  result <- unlist(result[fit.measures])
   class(result) <- c("lavaan.vector","numeric")
-  unlist(result[fit.measures])
+  result
 }
 
 
@@ -209,7 +331,7 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
 ##' Calculate the RMSEA of the null (baseline) model
 ##'
 ##' RMSEA of the null model is calculated similar to the formula provided in the
-##' \code{lavaan} package. The standard formula of RMSEA is
+##' `lavaan` package. The standard formula of RMSEA is
 ##'
 ##' \deqn{ RMSEA =\sqrt{\frac{\chi^2}{N \times df} - \frac{1}{N}} \times
 ##' \sqrt{G} }
@@ -225,18 +347,18 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
 ##' RMSEA for the null model is less than 0.158, an incremental measure of fit
 ##' may not be that informative."
 ##'
-##' See also \url{http://davidakenny.net/cm/fit.htm}
+##' See also the paper cited in **References**.
 ##'
 ##'
 ##' @importFrom lavaan lavInspect
 ##'
-##' @param object The lavaan model object provided after running the \code{cfa},
-##' \code{sem}, \code{growth}, or \code{lavaan} functions.
-##' @param scaled If \code{TRUE}, the scaled (or robust, if available) RMSEA
+##' @param object The lavaan model object provided after running the `cfa`,
+##' `sem`, `growth`, or `lavaan` functions.
+##' @param scaled If `TRUE`, the scaled (or robust, if available) RMSEA
 ##'   is returned. Ignored if a robust test statistic was not requested.
-##' @param silent If \code{TRUE}, do not print anything on the screen.
+##' @param silent If `TRUE`, do not print anything on the screen.
 ##'
-##' @return A value of RMSEA of the null model (a \code{numeric} vector)
+##' @return A value of RMSEA of the null model (a `numeric` vector)
 ##'   returned invisibly.
 ##'
 ##' @author
@@ -246,14 +368,14 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
 ##'
 ##' @seealso
 ##' \itemize{
-##'   \item \code{\link{miPowerFit}} For the modification indices and their
+##'   \item [miPowerFit()] For the modification indices and their
 ##'      power approach for model fit evaluation
-##'   \item \code{\link{moreFitIndices}} For other fit indices
+##'   \item [moreFitIndices()] For other fit indices
 ##' }
 ##'
 ##' @references Kenny, D. A., Kaniskan, B., & McCoach, D. B. (2015). The
 ##' performance of RMSEA in models with small degrees of freedom.
-##' \emph{Sociological Methods Research, 44}(3), 486--507.
+##' *Sociological Methods Research, 44*(3), 486--507.
 ##' \doi{10.1177/0049124114543236}
 ##'
 ##' @examples
@@ -267,7 +389,8 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
 ##'
 ##' @export
 nullRMSEA <- function(object, scaled = FALSE, silent = FALSE) {
-  fit <- lavaan::update(object, model = lavaan::lav_partable_independence(object))
+  fit <- lavaan::update(object, slotData = object@Data,
+                        model = lavaan::lav_partable_independence(object))
 	fits <- lavaan::fitMeasures(fit, fit.measures = c("rmsea","rmsea.scaled",
 	                                                  "rmsea.robust"))
 	if (scaled) {
@@ -311,51 +434,49 @@ sic <- function(f, lresults = NULL) {
   if (DET <= 0) return(NA)
 
   ## return SIC
-  f + log(DET)
+
 }
 
 
 
-##' Small-\emph{N} correction for \eqn{chi^2} test statistic
+##' Small-*N* correction for \eqn{chi^2} test statistic
 ##'
-##' Calculate small-\emph{N} corrections for \eqn{chi^2} model-fit test
+##' Calculate small-*N* corrections for \eqn{chi^2} model-fit test
 ##' statistic to adjust for small sample size (relative to model size).
 ##'
 ##' Four finite-sample adjustments to the chi-squared statistic are currently
 ##' available, all of which are described in Shi et al. (2018). These all
 ##' assume normally distributed data, and may not work well with severely
-##' nonnormal data. Deng et al. (2018, section 4) review proposed small-\emph{N}
+##' nonnormal data. Deng et al. (2018, section 4) review proposed small-*N*
 ##' adjustments that do not assume normality, which rarely show promise, so
 ##' they are not implemented here. This function currently will apply
-##' small-\emph{N} adjustments to scaled test statistics with a warning that
+##' small-*N* adjustments to scaled test statistics with a warning that
 ##' they do not perform well (Deng et al., 2018).
 ##'
 ##' @importFrom lavaan lavInspect lavNames
 ##' @importFrom stats pchisq
 ##' @importFrom methods getMethod
 ##'
-##' @param fit0,fit1 \linkS4class{lavaan} object(s) provided after running the
-##'   \code{cfa}, \code{sem}, \code{growth}, or \code{lavaan} functions.
-##'   \linkS4class{lavaan.mi} object(s) also accepted.
-##' @param smallN.method \code{character} indicating the small-\emph{N}
+##' @param fit0,fit1 [lavaan::lavaan-class] or [lavaan.mi::lavaan.mi-class] object(s)
+##' @param smallN.method `character` indicating the small-*N*
 ##'   correction method to use. Multiple may be chosen (all of which assume
 ##'   normality), as described in Shi et al. (2018):
-##'   \code{c("swain","yuan.2015","yuan.2005","bartlett")}. Users may also
-##'   simply select \code{"all"}.
-##' @param \dots Additional arguments to the \code{\link[lavaan]{lavTestLRT}} or
-##'   \code{\link{lavTestLRT.mi}} functions. Ignored when \code{is.null(fit1)}.
-##' @param omit.imps \code{character} vector specifying criteria for omitting
-##'   imputations from pooled results. Ignored unless \code{fit0} (and
-##'   optionally \code{fit1}) is a \linkS4class{lavaan.mi} object. See
-##'   \code{\link{lavTestLRT.mi}} for a description of options and defaults.
+##'   `c("swain","yuan.2015","yuan.2005","bartlett")`. Users may also
+##'   simply select `"all"`.
+##' @param \dots Additional arguments to the [lavaan::lavTestLRT()] or
+##'   [lavaan.mi::lavTestLRT.mi()] functions. Ignored when `is.null(fit1)`.
+##' @param omit.imps `character` vector specifying criteria for omitting
+##'   imputations from pooled results. Ignored unless `fit0` (and
+##'   optionally `fit1`) is a [lavaan.mi::lavaan.mi-class] object. See
+##'   [lavaan.mi::lavTestLRT.mi()] for a description of options and defaults.
 ##'
-##' @return A \code{list} of \code{numeric} vectors: one for the originally
-##'   requested statistic(s), along with one per requested \code{smallN.method}.
-##'   All include the the (un)adjusted test statistic, its \emph{df}, and the
-##'   \emph{p} value for the test under the null hypothesis that the model fits
+##' @return A `list` of `numeric` vectors: one for the originally
+##'   requested statistic(s), along with one per requested `smallN.method`.
+##'   All include the the (un)adjusted test statistic, its *df*, and the
+##'   *p* value for the test under the null hypothesis that the model fits
 ##'   perfectly (or that the 2 models have equivalent fit).
 ##'   The adjusted chi-squared statistic(s) also include(s) the scaling factor
-##'   for the small-\emph{N} adjustment.
+##'   for the small-*N* adjustment.
 ##'
 ##' @author
 ##'   Terrence D. Jorgensen (University of Amsterdam; \email{TJorgensen314@@gmail.com})
@@ -363,12 +484,12 @@ sic <- function(f, lresults = NULL) {
 ##' @references
 ##'   Deng, L., Yang, M., & Marcoulides, K. M. (2018). Structural equation
 ##'   modeling with many variables: A systematic review of issues and
-##'   developments. \emph{Frontiers in Psychology, 9}, 580.
+##'   developments. *Frontiers in Psychology, 9*, 580.
 ##'   \doi{10.3389/fpsyg.2018.00580}
 ##'
 ##'   Shi, D., Lee, T., & Terry, R. A. (2018). Revisiting the model
 ##'   size effect in structural equation modeling.
-##'   \emph{Structural Equation Modeling, 25}(1), 21--40.
+##'   *Structural Equation Modeling, 25*(1), 21--40.
 ##'   \doi{10.1080/10705511.2017.1369088}
 ##'
 ##' @examples
@@ -378,12 +499,12 @@ sic <- function(f, lresults = NULL) {
 ##'     textual =~ x4 + b2*x5 + x6
 ##'     speed   =~ x7 + b3*x8 + x9
 ##' '
-##' fit1 <- cfa(HS.model, data = HolzingerSwineford1939[1:50,])
+##' fit1 <- cfa(HS.model, data = HolzingerSwineford1939[101:150,])
 ##' ## test a single model (implicitly compared to a saturated model)
 ##' chisqSmallN(fit1)
 ##'
 ##' ## fit a more constrained model
-##' fit0 <- cfa(HS.model, data = HolzingerSwineford1939[1:50,],
+##' fit0 <- cfa(HS.model, data = HolzingerSwineford1939[101:150,],
 ##'             orthogonal = TRUE)
 ##' ## compare 2 models
 ##' chisqSmallN(fit1, fit0)
@@ -400,24 +521,36 @@ chisqSmallN <- function(fit0, fit1 = NULL,
     stop('No recognized options for "smallN.method" argument')
 
   ## check class
-  if (!inherits(fit0, what = c("lavaan","lavaanList")))
-    stop("this function is only applicable to fitted lavaan models")
+  if (!inherits(fit0, what = c("lavaan","lavaan.mi")))
+    stop("this function is only applicable to fitted lavaan(.mi) models")
+
+  ## necessary to load lavaan.mi?
+  if (inherits(fit0, what = c("lavaan.mi"))) {
+    if (!"package:lavaan.mi" %in% search()) attachNamespace("lavaan.mi")
+  }
 
   ## if there are 2 models...
   if (!is.null(fit1)) {
 
     ## check classes
-    if (!inherits(fit1, what = c("lavaan","lavaanList")))
-      stop("this function is only applicable to fitted lavaan models")
+    if (!inherits(fit1, what = c("lavaan","lavaan.mi")))
+      stop("this function is only applicable to fitted lavaan(.mi) models")
     modClass <- unique(sapply(list(fit0, fit1), class))
     if (length(modClass) > 1L) stop('All models must be of the same class (e.g.,',
                                     ' cannot compare lavaan objects to lavaan.mi)')
 
     ## check order of DF
-    suppressMessages(DF0 <- getMethod("fitMeasures", class(fit0))(fit0, fit.measures = "df",
-                                                                  omit.imps = omit.imps)[1])
-    suppressMessages(DF1 <- getMethod("fitMeasures", class(fit1))(fit1, fit.measures = "df",
-                                                                  omit.imps = omit.imps)[1])
+    # suppressMessages(DF0 <- getMethod("fitMeasures", class(fit0))(fit0, fit.measures = "df",
+    #                                                               omit.imps = omit.imps)[1])
+    # suppressMessages(DF1 <- getMethod("fitMeasures", class(fit1))(fit1, fit.measures = "df",
+    #                                                               omit.imps = omit.imps)[1])
+    if (inherits(fit0, "lavaan.mi")) {
+      DF0 <- fitMeasures(fit0, fit.measures = "df", omit.imps = omit.imps)[1]
+      DF1 <- fitMeasures(fit1, fit.measures = "df", omit.imps = omit.imps)[1]
+    } else {
+      DF0 <- fitMeasures(fit0, fit.measures = "df")[1]
+      DF1 <- fitMeasures(fit1, fit.measures = "df")[1]
+    }
     if (DF0 == DF1) stop("Models have the same degrees of freedom.")
     parent <- which.min(c(DF0, DF1))
     if (parent == 1L) {
@@ -441,7 +574,7 @@ chisqSmallN <- function(fit0, fit1 = NULL,
   if (!is.null(fit1)) {
 
     if (any(smallN.method %in% c("yuan.2015","swain"))) {
-      message('Swain(1975) and Yuan (2015) corrections depend on the number ',
+      message('Swain (1975) and Yuan (2015) corrections depend on the number ',
               'of free parameters, so it is unavailable for model comparison.')
       smallN.method <- smallN.method[-which(smallN.method %in% c("yuan.2015","swain"))]
     }
@@ -461,13 +594,19 @@ chisqSmallN <- function(fit0, fit1 = NULL,
   K <- length(lavNames(fit0, type = "lv")) # count latent factors
 
   if (is.null(fit1)) {
-    FIT <- getMethod("fitMeasures", class(fit0))(fit0,
-                                                 ## lavaan.mi arguments ignored
-                                                 ## for lavaan objects
-                                                 omit.imps = omit.imps,
-                                                 asymptotic = TRUE,
-                                                 fit.measures = c("npar","chisq",
-                                                                  "df","pvalue"))
+    # FIT <- getMethod("fitMeasures", class(fit0))(fit0,
+    #                                              ## lavaan.mi arguments ignored
+    #                                              ## for lavaan objects
+    #                                              omit.imps = omit.imps,
+    #                                              asymptotic = TRUE,
+    #                                              fit.measures = c("npar","chisq",
+    #                                                               "df","pvalue"))
+    if (inherits(fit0, "lavaan.mi")) {
+      FIT <- fitMeasures(fit0, fit.measures = c("npar","chisq","df","pvalue"),
+                         omit.imps = omit.imps, asymptotic = TRUE)
+    } else {
+      FIT <- fitMeasures(fit0, fit.measures = c("npar","chisq","df","pvalue"))
+    }
     scaled <- any(grepl(pattern = "scaled", x = names(FIT)))
     if (scaled) warning('Small-N corrections developed assuming normality, but',
                         ' a scaled test was requested. Applying correction(s) ',
@@ -494,7 +633,7 @@ chisqSmallN <- function(fit0, fit1 = NULL,
       K <- max(K, K1)
     }
 
-    try(AOV <- compareFit(fit0, fit1, argsLRT = list(...), indices = FALSE),
+    AOV <- try(compareFit(fit0, fit1, argsLRT = list(...), indices = FALSE),
         silent = TRUE)
     if (inherits(AOV, "try-error")) stop('Model comparison failed. Try using ',
                                          'lavTestLRT() to investigate why.')
