@@ -1,5 +1,5 @@
 ### Terrence D. Jorgensen
-### Last updated: 17 September 2018
+### Last updated: 9 February 2026
 ### Savalei & Yuan's (2009) model-based bootstrap for missing data
 
 
@@ -17,20 +17,20 @@
 ##' hist,BootMiss-method
 ##' @docType class
 ##' @section Objects from the Class: Objects can be created via the
-##' \code{\link{bsBootMiss}} function.
-##' @slot time A list containing 2 \code{difftime} objects (\code{transform}
-##'  and \code{fit}), indicating the time elapsed for data transformation and
+##' [bsBootMiss()] function.
+##' @slot time A list containing 2 `difftime` objects (`transform`
+##'  and `fit`), indicating the time elapsed for data transformation and
 ##'  for fitting the model to bootstrap data sets, respectively.
 ##' @slot transData Transformed data
 ##' @slot bootDist The vector of \eqn{chi^2} values from bootstrap data sets
 ##'  fitted by the target model
 ##' @slot origChi The \eqn{chi^2} value from the original data set
 ##' @slot df The degree of freedom of the model
-##' @slot bootP The \emph{p} value comparing the original \eqn{chi^2} with the
+##' @slot bootP The *p* value comparing the original \eqn{chi^2} with the
 ##'  bootstrap distribution
 ##' @author Terrence D. Jorgensen (University of Amsterdam;
 ##' \email{TJorgensen314@@gmail.com})
-##' @seealso \code{\link{bsBootMiss}}
+##' @seealso [bsBootMiss()]
 ##' @examples
 ##'
 ##' # See the example from the bsBootMiss function
@@ -80,18 +80,18 @@ function(object) {
 ##' @rdname BootMiss-class
 ##' @aliases hist,BootMiss-method
 ##' @importFrom stats qchisq dchisq quantile
-##' @param object,x object of class \code{BootMiss}
-##' @param ... Additional arguments to pass to \code{\link[graphics]{hist}}
+##' @param object,x object of class `BootMiss`
+##' @param ... Additional arguments to pass to [graphics::hist()]
 ##' @param alpha alpha level used to draw confidence limits
 ##' @param nd number of digits to display
-##' @param printLegend \code{logical}. If \code{TRUE} (default), a legend will
+##' @param printLegend `logical`. If `TRUE` (default), a legend will
 ##'  be printed with the histogram
-##' @param legendArgs \code{list} of arguments passed to the
-##'  \code{\link[graphics]{legend}} function.  The default argument is a list
+##' @param legendArgs `list` of arguments passed to the
+##'  [graphics::legend()] function.  The default argument is a list
 ##'  placing the legend at the top-left of the figure.
-##' @return The \code{hist} method returns a list of \code{length == 2},
-##'  containing the arguments for the call to \code{hist} and the arguments
-##'  to the call for \code{legend}, respectively.
+##' @return The `hist` method returns a list of `length == 2`,
+##'  containing the arguments for the call to `hist` and the arguments
+##'  to the call for `legend`, respectively.
 ##' @export
 setMethod("hist", "BootMiss",
 function(x, ..., alpha = .05, nd = 2, printLegend = TRUE,
@@ -121,11 +121,11 @@ function(x, ..., alpha = .05, nd = 2, printLegend = TRUE,
                      " same precision as your requested alpha level (alpha = ",
                      alpha, ")"))
     }
-    pVal <- round(x@bootP, nd)
+
     if (x@bootP < (1 / 10^nd)) {
       pVal <- paste(c("< .", rep(0, nd - 1),"1"), collapse = "")
     } else {
-      paste("=", round(x@bootP, nd))
+      pVal <- paste("=", round(x@bootP, nd))
     }
 
     if (is.null(legendArgs$box.lty)) legendArgs$box.lty <- 0
@@ -166,87 +166,90 @@ function(x, ..., alpha = .05, nd = 2, printLegend = TRUE,
 ##' Implement the Bollen and Stine's (1992) Bootstrap when missing observations
 ##' exist. The implemented method is proposed by Savalei and Yuan (2009). This
 ##' can be used in two ways. The first and easiest option is to fit the model to
-##' incomplete data in \code{lavaan} using the FIML estimator, then pass that
-##' \code{lavaan} object to \code{bsBootMiss}.
+##' incomplete data in `lavaan` using the FIML estimator, then pass that
+##' `lavaan` object to `bsBootMiss`.
 ##'
 ##' The second is designed for users of other software packages (e.g., LISREL,
 ##' EQS, Amos, or Mplus). Users can import their data, \eqn{\chi^2} value, and
 ##' model-implied moments from another package, and they have the option of
 ##' saving (or writing to a file) either the transformed data or bootstrapped
 ##' samples of that data, which can be analyzed in other programs. In order to
-##' analyze the bootstrapped samples and return a \emph{p} value, users of other
+##' analyze the bootstrapped samples and return a *p* value, users of other
 ##' programs must still specify their model using lavaan syntax.
 ##'
 ##'
 ##' @importFrom lavaan lavInspect parTable
-##' @param x A target \code{lavaan} object used in the Bollen-Stine bootstrap
+##' @param x A target `lavaan` object used in the Bollen-Stine bootstrap
 ##' @param transformation The transformation methods in Savalei and Yuan (2009).
 ##' There are three methods in the article, but only the first two are currently
-##' implemented here.  Use \code{transformation = 1} when there are few missing
+##' implemented here.  Use `transformation = 1` when there are few missing
 ##' data patterns, each of which has a large size, such as in a
-##' planned-missing-data design.  Use \code{transformation = 2} when there are
+##' planned-missing-data design.  Use `transformation = 2` when there are
 ##' more missing data patterns. The currently unavailable
-##' \code{transformation = 3} would be used when several missing data patterns
+##' `transformation = 3` would be used when several missing data patterns
 ##' have n = 1.
 ##' @param nBoot The number of bootstrap samples.
-##' @param model Optional. The target model if \code{x} is not provided.
-##' @param rawData Optional. The target raw data set if \code{x} is not
+##' @param model Optional. The target model if `x` is not provided.
+##' @param rawData Optional. The target raw data set if `x` is not
 ##'  provided.
-##' @param Sigma Optional. The model-implied covariance matrix if \code{x} is
+##' @param Sigma Optional. The model-implied covariance matrix if `x` is
 ##'  not provided.
-##' @param Mu Optional. The model-implied mean vector if \code{x} is not
+##' @param Mu Optional. The model-implied mean vector if `x` is not
 ##'  provided.
 ##' @param group Optional character string specifying the name of the grouping
-##'  variable in \code{rawData} if \code{x} is not provided.
+##'  variable in `rawData` if `x` is not provided.
 ##' @param ChiSquared Optional. The model's \eqn{\chi^2} test statistic if
-##'  \code{x} is not provided.
-##' @param EMcov Optional, if \code{x} is not provided. The EM (or Two-Stage ML)
+##'  `x` is not provided.
+##' @param EMcov Optional, if `x` is not provided. The EM (or Two-Stage ML)
 ##' estimated covariance matrix used to speed up Transformation 2 algorithm.
-##' @param transDataOnly Logical. If \code{TRUE}, the result will provide the
+##' @param transDataOnly Logical. If `TRUE`, the result will provide the
 ##' transformed data only.
-##' @param writeTransData Logical. If \code{TRUE}, the transformed data set is
-##' written to a text file, \code{transDataOnly} is set to \code{TRUE}, and the
+##' @param writeTransData Logical. If `TRUE`, the transformed data set is
+##' written to a text file, `transDataOnly` is set to `TRUE`, and the
 ##' transformed data is returned invisibly.
-##' @param bootSamplesOnly Logical. If \code{TRUE}, the result will provide
+##' @param bootSamplesOnly Logical. If `TRUE`, the result will provide
 ##' bootstrap data sets only.
-##' @param writeBootData Logical. If \code{TRUE}, the stacked bootstrap data
-##' sets are written to a text file, \code{bootSamplesOnly} is set to
-##' \code{TRUE}, and the list of bootstrap data sets are returned invisibly.
-##' @param writeArgs Optional \code{list}. If \code{writeBootData = TRUE} or
-##' \code{writeBootData = TRUE}, user can pass arguments to the
-##' \code{\link[utils]{write.table}} function as a list.  Some default values
-##' are provided: \code{file} = "bootstrappedSamples.dat", \code{row.names} =
-##' \code{FALSE}, and \code{na} = "-999", but the user can override all of these
-##' by providing other values for those arguments in the \code{writeArgs} list.
+##' @param writeBootData Logical. If `TRUE`, the stacked bootstrap data
+##' sets are written to a text file, `bootSamplesOnly` is set to
+##' `TRUE`, and the list of bootstrap data sets are returned invisibly.
+##' @param writeArgs Optional `list`. If `writeBootData = TRUE` or
+##' `writeBootData = TRUE`, user can pass arguments to the
+##' [utils::write.table()] function as a list.  Some default values
+##' are provided: `file` = "bootstrappedSamples.dat", `row.names` =
+##' `FALSE`, and `na` = "-999", but the user can override all of these
+##' by providing other values for those arguments in the `writeArgs` list.
 ##' @param seed The seed number used in randomly drawing bootstrap samples.
-##' @param suppressWarn Logical. If \code{TRUE}, warnings from \code{lavaan}
+##' @param suppressWarn Logical. If `TRUE`, warnings from `lavaan`
 ##' function will be suppressed when fitting the model to each bootstrap sample.
 ##' @param showProgress Logical. Indicating whether to display a progress bar
 ##' while fitting models to bootstrap samples.
-##' @param \dots The additional arguments in the \code{\link[lavaan]{lavaan}}
-##' function. See also \code{\link[lavaan]{lavOptions}}
-##' @return As a default, this function returns a \code{\linkS4class{BootMiss}}
-##' object containing the results of the bootstrap samples. Use \code{show},
-##' \code{summary}, or \code{hist} to examine the results. Optionally, the
-##' transformed data set is returned if \code{transDataOnly = TRUE}. Optionally,
-##' the bootstrap data sets are returned if \code{bootSamplesOnly = TRUE}.
+##' @param \dots The additional arguments in the [lavaan::lavaan()]
+##' function. See also [lavaan::lavOptions()]
+##' @return As a default, this function returns a [BootMiss-class]
+##' object containing the results of the bootstrap samples. Use `show`,
+##' `summary`, or `hist` to examine the results. Optionally, the
+##' transformed data set is returned if `transDataOnly = TRUE`. Optionally,
+##' the bootstrap data sets are returned if `bootSamplesOnly = TRUE`.
 ##' @author Terrence D. Jorgensen (University of Amsterdam;
 ##' \email{TJorgensen314@@gmail.com})
 ##'
-##' Syntax for transformations borrowed from http://www2.psych.ubc.ca/~vsavalei/
-##' @seealso \code{\linkS4class{BootMiss}}
+##' Syntax for transformations borrowed from supplementary materials
+##' in Savalei & Yuan (2009)
+##'
+##' @seealso [BootMiss-class]
+##'
 ##' @references
 ##'
 ##' Bollen, K. A., & Stine, R. A. (1992). Bootstrapping goodness-of-fit measures
-##' in structural equation models. \emph{Sociological Methods &
-##' Research, 21}(2), 205--229. doi:10.1177/0049124192021002004
+##' in structural equation models. *Sociological Methods &
+##' Research, 21*(2), 205--229. \doi{10.1177/0049124192021002004}
 ##'
 ##' Savalei, V., & Yuan, K.-H. (2009). On the model-based bootstrap with missing
-##' data: Obtaining a p-value for a test of exact fit. \emph{Multivariate
-##' Behavioral Research, 44}(6), 741--763. doi:10.1080/00273170903333590
+##' data: Obtaining a p-value for a test of exact fit. *Multivariate
+##' Behavioral Research, 44*(6), 741--763. \doi{10.1080/00273170903333590}
+##'
 ##' @examples
 ##'
-##' \dontrun{
 ##' dat1 <- HolzingerSwineford1939
 ##' dat1$x5 <- ifelse(dat1$x1 <= quantile(dat1$x1, .3), NA, dat1$x5)
 ##' dat1$x9 <- ifelse(is.na(dat1$x5), NA, dat1$x9)
@@ -260,7 +263,8 @@ function(x, ..., alpha = .05, nd = 2, printLegend = TRUE,
 ##'                  missing = "fiml", group = "school")
 ##' summary(targetFit, fit = TRUE, standardized = TRUE)
 ##'
-##' # The number of bootstrap samples should be much higher.
+##' \donttest{
+##' ## The number of bootstrap samples should be much higher than this example
 ##' temp <- bsBootMiss(targetFit, transformation = 1, nBoot = 10, seed = 31415)
 ##'
 ##' temp
@@ -347,20 +351,20 @@ bsBootMiss <- function(x, transformation = 2, nBoot = 500, model, rawData,
     }
 
     ## If user supplies one-group data & moments, convert to lists.
-    if (class(rawData) == "data.frame") rawData <- list(rawData)
-    if (class(rawData) != "list") {
+    if ( inherits(rawData, "data.frame")) rawData <- list(rawData)
+    if (!inherits(rawData, "list")) {
       stop("The \"rawData\" argument must be a data.frame or list of data frames.")
     } else {
       if (!all(sapply(rawData, is.data.frame))) stop("Every element of \"rawData\" must be a data.frame")
     }
-    if (class(Sigma) == "matrix") Sigma <- list(Sigma)
+    if (inherits(Sigma, "matrix")) Sigma <- list(Sigma)
     if (is.numeric(Mu)) Mu <- list(Mu)
 
     ## check whether EMcov was supplied for starting values in Trans2/Trans3
     if (!hasArg(EMcov)) {
       EMcov <- vector("list", length(Sigma))
     } else {
-      if (class(EMcov) == "matrix") EMcov <- list(EMcov)
+      if (inherits(EMcov, "matrix")) EMcov <- list(EMcov)
       ## check EMcov is symmetric and dimensions match Sigma
       for (g in seq_along(EMcov)) {
         if (!isSymmetric(EMcov[[g]])) stop("EMcov in group ", g, " not symmetric.")
@@ -720,7 +724,7 @@ trans3 <- function(dat, Sigma, Mu, EMcov) {
   Tj3add <- matrix(0, nrow = p, ncol = p * p)
 
   ## Create Duplication Matrix and its inverse (Magnus & Neudecker, 1999)
-  Dup <- lavaan::duplicationMatrix(p)
+  Dup <- lavaan::lav_matrix_duplication(p)
   Dupinv <- solve(t(Dup) %*% Dup) %*% t(Dup)
 
   ## step through each MD pattern, populate Hjs and Mjs
